package function.builtin.math;

import java.math.BigInteger;

import function.*;
import sexpression.*;

public class DIVIDE extends LispFunction {

    private ArgumentValidator argumentValidator;
    private MathFunction mathFunction;

    public DIVIDE() {
        this.argumentValidator = new ArgumentValidator("/");
        this.argumentValidator.setMinimumNumberOfArguments(1);
        this.argumentValidator.setEveryArgumentExpectedType(LispNumber.class);
        this.mathFunction = new MathFunction(this::getReciprocal, this::divide);
    }

    public LispNumber call(Cons argumentList) {
        argumentValidator.validate(argumentList);

        return mathFunction.callTailRecursive(argumentList);
    }

    private LispNumber getReciprocal(LispNumber number1) {
        return new LispNumber(BigInteger.ONE.divide(number1.getValue()));
    }

    private LispNumber divide(LispNumber number1, LispNumber number2) {
        return new LispNumber(number1.getValue().divide(number2.getValue()));
    }

}
