package function.builtin.predicate;

import function.*;
import sexpression.*;

public class EQ extends LispFunction {

    private ArgumentValidator argumentValidator;

    public EQ() {
        this.argumentValidator = new ArgumentValidator("EQ");
        this.argumentValidator.setExactNumberOfArguments(2);
    }

    public SExpression call(Cons argumentList) {
        argumentValidator.validate(argumentList);

        Cons cdr = (Cons) argumentList.getCdr();
        SExpression firstArgument = argumentList.getCar();
        SExpression secondArgument = cdr.getCar();

        return eq(firstArgument, secondArgument);
    }

    private SExpression eq(SExpression firstArgument, SExpression secondArgument) {
        if (isAtomPair(firstArgument, secondArgument))
            return atomEq(firstArgument, secondArgument);

        return listEq(firstArgument, secondArgument);
    }

    private boolean isAtomPair(SExpression firstArgument, SExpression secondArgument) {
        return firstArgument.atomp() && secondArgument.atomp();
    }

    private SExpression atomEq(SExpression firstArgument, SExpression secondArgument) {
        return (firstArgument.toString().equals(secondArgument.toString())) ? Symbol.T : Nil.getInstance();
    }

    private SExpression listEq(SExpression firstArgument, SExpression secondArgument) {
        return (firstArgument == secondArgument) ? Symbol.T : Nil.getInstance();
    }

}
