package function.builtin.special;

import java.util.HashMap;

import function.*;
import function.builtin.EVAL;
import function.builtin.cons.LIST;
import sexpression.*;

public class DEFUN extends LispFunction {

    private ArgumentValidator argumentValidator;
    private ArgumentValidator lambdaListIsListValidator;
    private ArgumentValidator lambdaListValidator;

    public DEFUN() {
        this.argumentValidator = new ArgumentValidator("DEFUN");
        this.argumentValidator.setMinimumNumberOfArguments(3);
        this.argumentValidator.setFirstArgumentExpectedType(Symbol.class);

        this.lambdaListIsListValidator = new ArgumentValidator("DEFUN|lambda_list|");
        this.lambdaListIsListValidator.setEveryArgumentExpectedType(Cons.class);

        this.lambdaListValidator = new ArgumentValidator("DEFUN|lambda_list|");
        this.lambdaListValidator.setEveryArgumentExpectedType(Symbol.class);
    }

    public SExpression call(Cons argumentList) {
        argumentValidator.validate(argumentList);

        Cons remainingArguments = (Cons) argumentList.getCdr();
        SExpression functionName = argumentList.getCar();
        SExpression secondArgument = remainingArguments.getCar();
        lambdaListIsListValidator.validate(LIST.makeList(secondArgument));

        Cons lambdaList = (Cons) secondArgument;
        lambdaListValidator.validate(lambdaList);

        Cons functionBody = (Cons) remainingArguments.getCdr();
        HashMap<String, LispFunction> functionTable = EVAL.getFunctionTable();

        if (functionTable.containsKey(functionName.toString()))
            System.out.println("WARNING: redefining function " + functionName.toString());

        functionTable.put(functionName.toString(), new UserDefinedFunction(functionName.toString(), lambdaList, functionBody));

        return functionName;
    }

    public boolean evaluateArguments() {
        return false;
    }

}
