package interpreter;

import java.io.*;
import java.text.MessageFormat;

import environment.Environment;
import error.*;
import function.builtin.EVAL;
import parser.LispParser;
import sexpression.SExpression;

public class LispInterpreter {

    public static final String ANSI_RESET = "\u001B[0m";
    public static final String ANSI_GREEN = "\u001B[32m";

    private LispParser parser;
    private ErrorManager errorManager;
    protected Environment environment;

    public LispInterpreter() {
        this.environment = Environment.getInstance();
        this.errorManager = new ErrorManager();
        this.parser = new LispParser(this.environment.getInput(), this.environment.getInputName());
    }

    public LispInterpreter(String fileName) {
        this.environment = Environment.getInstance();
        this.errorManager = new ErrorManager();

        try {
            this.environment.setInput(new FileInputStream(fileName));
        } catch (FileNotFoundException e) {
            this.errorManager.generateError(new LispFileNotFoundException(e));
        }

        this.parser = new LispParser(this.environment.getInput(), this.environment.getInputName());
    }

    public void interpret() {
        printGreeting();

        for (displayPrompt(); !parser.isEof(); displayPrompt())
            printValueOfNextSExpression();

        printFarewell();
    }

    protected void printGreeting() {}

    protected void displayPrompt() {}

    private void printValueOfNextSExpression() {
        try {
            printValueOfNextSExpressionWithException();
        } catch (LispException e) {
            erasePrompt();
            errorManager.generateError(e);
        } catch (RuntimeException e) {
            erasePrompt();
            errorManager.generateError(convertToLispException(e));
        }
    }

    private void printValueOfNextSExpressionWithException() {
        SExpression sExpression = parser.getNextSExpression();
        String result = MessageFormat.format("{0}{1}{2}", ANSI_GREEN, EVAL.eval(sExpression), ANSI_RESET);

        erasePrompt();
        environment.getOutput().println(result);
    }

    protected void erasePrompt() {}

    private LispException convertToLispException(RuntimeException e) {
        return new LispException() {

            private static final long serialVersionUID = 1L;

            @Override
            public String getMessage() {
                return e.getMessage();
            }
        };
    }

    protected void printFarewell() {
        environment.getOutput().println();
    }

    public static class LispFileNotFoundException extends LispException {

        private static final long serialVersionUID = 1L;
        private String message;

        public LispFileNotFoundException(FileNotFoundException e) {
            this.message = e.getMessage();
        }

        @Override
        public int getSeverity() {
            return ErrorManager.CRITICAL_LEVEL;
        }

        @Override
        public String getMessage() {
            return message;
        }
    }

}
